@@ -22,9 +22,6 @@
 
 #include <stdlib.h>
 
-#include <glade/glade.h>
-#include <glade/glade-build.h>
-
 /* Regular expressions */
 #define CTL	"\\x00-\\x1f\\x7f"
 #define SEP	"\\x09\\x20\\(\\)<>@,;:\\\\\"/\\[\\]\\?=\\{\\}"
@@ -187,40 +184,32 @@ gtkhtml_editor_private_init (GtkhtmlEditor *editor)
 	priv->spell_suggestion_menus =
 		g_hash_table_new (g_direct_hash, g_direct_equal);
 
-	filename = gtkhtml_editor_find_data_file ("gtkhtml-editor.ui");
-	gtk_ui_manager_add_ui_from_file (priv->manager, filename, &error);
-	g_free (filename);
+	filename = gtkhtml_editor_find_data_file ("gtkhtml-editor-manager.ui");
 
-	if (error != NULL) {
-		/* Henceforth, bad things start happening. */
-		g_critical ("%s", error->message);
+	if (!gtk_ui_manager_add_ui_from_file (priv->manager, filename, &error)) {
+		g_critical ("Couldn't load builder file: %s\n", error->message);
 		g_clear_error (&error);
 	}
 
+	g_free (filename);
+
+	filename = gtkhtml_editor_find_data_file ("gtkhtml-editor-builder.ui");
+
+	priv->builder = gtk_builder_new ();
+	if (!gtk_builder_add_from_file (priv->builder, filename, &error)) {
+		g_critical ("Couldn't load builder file: %s\n", error->message);
+		g_clear_error (&error);
+	}
+
+	g_free (filename);
+
 	gtkhtml_editor_actions_init (editor);
 
 	gtk_window_add_accel_group (
 		GTK_WINDOW (editor),
 		gtk_ui_manager_get_accel_group (priv->manager));
 
-	glade_register_widget (
-		GTKHTML_TYPE_COLOR_COMBO,
-		glade_standard_build_widget, NULL, NULL);
-
-	glade_register_widget (
-		GTKHTML_TYPE_COLOR_SWATCH,
-		glade_standard_build_widget, NULL, NULL);
-
-	glade_register_widget (
-		GTKHTML_TYPE_COMBO_BOX,
-		glade_standard_build_widget, NULL, NULL);
-
-	glade_provide ("gtkhtml-editor");
-
-	filename = gtkhtml_editor_find_data_file ("gtkhtml-editor.glade");
-	priv->glade_xml = glade_xml_new (filename, NULL, NULL);
-	glade_xml_signal_autoconnect (priv->glade_xml);
-	g_free (filename);
+	gtk_builder_connect_signals (priv->builder, NULL);
 
 	/* Construct main window widgets. */
 
@@ -235,13 +224,13 @@ gtkhtml_editor_private_init (GtkhtmlEditor *editor)
 	gtk_widget_show (widget);
 
 	widget = gtkhtml_editor_get_managed_widget (editor, "/edit-toolbar");
-	gtk_toolbar_set_style (GTK_TOOLBAR (widget), GTK_TOOLBAR_ICONS);
+	gtk_toolbar_set_style (GTK_TOOLBAR (widget), GTK_TOOLBAR_BOTH_HORIZ);
 	gtk_box_pack_start (GTK_BOX (editor->vbox), widget, FALSE, FALSE, 0);
 	priv->edit_toolbar = g_object_ref (widget);
 	gtk_widget_show (widget);
 
 	widget = gtkhtml_editor_get_managed_widget (editor, "/html-toolbar");
-	gtk_toolbar_set_style (GTK_TOOLBAR (widget), GTK_TOOLBAR_ICONS);
+	gtk_toolbar_set_style (GTK_TOOLBAR (widget), GTK_TOOLBAR_BOTH_HORIZ);
 	gtk_box_pack_start (GTK_BOX (editor->vbox), widget, FALSE, FALSE, 0);
 	priv->html_toolbar = g_object_ref (widget);
 	gtk_widget_show (widget);
@@ -250,6 +239,8 @@ gtkhtml_editor_private_init (GtkhtmlEditor *editor)
 	gtk_scrolled_window_set_policy (
 		GTK_SCROLLED_WINDOW (widget),
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type (
+		GTK_SCROLLED_WINDOW (widget), GTK_SHADOW_IN);
 	gtk_box_pack_start (GTK_BOX (editor->vbox), widget, TRUE, TRUE, 0);
 	priv->scrolled_window = g_object_ref (widget);
 	gtk_widget_show (widget);
@@ -387,7 +378,7 @@ gtkhtml_editor_private_dispose (GtkhtmlEditor *editor)
 	DISPOSE (priv->language_actions);
 	DISPOSE (priv->spell_check_actions);
 	DISPOSE (priv->suggestion_actions);
-	DISPOSE (priv->glade_xml);
+	DISPOSE (priv->builder);
 
 	DISPOSE (priv->html_painter);
 	DISPOSE (priv->plain_painter);
@@ -582,6 +573,40 @@ gtkhtml_editor_insert_file (GtkhtmlEditor *editor,
 	return response;
 }
 
+void
+gtkhtml_editor_show_uri (GtkWindow *parent,
+                         const gchar *uri)
+{
+	GtkWidget *dialog;
+	GdkScreen *screen = NULL;
+	GError *error = NULL;
+	guint32 timestamp;
+
+	g_return_if_fail (uri != NULL);
+
+	timestamp = gtk_get_current_event_time ();
+
+	if (parent != NULL)
+		screen = gtk_widget_get_screen (GTK_WIDGET (parent));
+
+	if (gtk_show_uri (screen, uri, timestamp, &error))
+		return;
+
+	dialog = gtk_message_dialog_new_with_markup (
+		parent, GTK_DIALOG_DESTROY_WITH_PARENT,
+		GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+		"<big><b>%s</b></big>",
+		_("Could not open the link."));
+
+	gtk_message_dialog_format_secondary_text (
+		GTK_MESSAGE_DIALOG (dialog), "%s", error->message);
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+
+	gtk_widget_destroy (dialog);
+	g_error_free (error);
+}
+
 void
 gtkhtml_editor_spell_check (GtkhtmlEditor *editor,
                             gboolean whole_document)
